@@ -67,6 +67,7 @@ generate a RuntimeError, and not a ProtocolError, which are unreadable
 to the user.
 """
 
+import errno
 import os, tempfile, shutil, subprocess, re
 from datetime import datetime
 
@@ -233,10 +234,10 @@ class Context():
         being ignored, it will skip it and look at the next line
         """
         line = fd.readline()
-        ignored = ('[GNUPG:] KEYEXPIRED', '[GNUPG:] SIGEXPIRED', 'gpg: ')
+        ignored = ('[GNUPG:] KEYEXPIRED', '[GNUPG:] SIGEXPIRED', '[GNUPG:] KEY_CONSIDERED', 'gpg: ')
 
         while line and line.startswith(ignored):
-            if self.debug: print >>self.debug, "SKIPPED:", line,
+            if self.debug: print >>self.debug, "IGNORED:", line,
             line = fd.readline()
 
         match = re.search(pattern, line)
@@ -310,6 +311,25 @@ class Keyring():
                 homedir = os.environ['GNUPGHOME']
         self.homedir = homedir
 
+    def get_agent_socket(self):
+        """get the location of the gpg-agent socket for this keyring"""
+        try:
+            command = ['gpgconf', '--homedir', self.homedir,
+                       '--list-dirs', 'agent-socket']
+            # silence errors
+            conf = subprocess.check_output(command,
+                                           stderr=subprocess.PIPE).rstrip()
+        # if we can't find gpgconf at all, assume the agent is in the
+        # homedir
+        except OSError as e:
+            if e.errno == errno.ENOENT:
+                conf = self.homedir + '/S.gpg-agent'
+        # GnuPG before 2.1.13 doesn't suport --homedir and will exit
+        # with an error. assume the agent is in the homedir in that
+        # case as well
+        except subprocess.CalledProcessError as e:
+            conf = self.homedir + '/S.gpg-agent'
+        return conf.rstrip()
 
     def import_data(self, data):
         """Import OpenPGP data blocks into the keyring.
@@ -530,7 +550,12 @@ class Keyring():
                 try:
                     self.context.seek(proc.stderr, 'GOOD_PASSPHRASE')
                 except GpgProtocolError:
-                    raise GpgRuntimeError(self.context.returncode, _('unable to prompt for passphrase, is gpg-agent running?'))
+                    # GnuPG 2.1 stopped outputing GOOD_PASSPHRASE, so try our luck at just waiting for output
+                    self.context.returncode = proc.wait()
+                    if self.context.returncode == 0:
+                        return True
+                    else:
+                        raise GpgRuntimeError(self.context.returncode, _('unable to prompt for passphrase, is gpg-agent running?'))
                 return proc.wait() == 0
 
             # don't sign all uids
@@ -569,15 +594,21 @@ class Keyring():
                 raise GpgRuntimeError(self.context.returncode, _('key is expired, cannot sign'))
             else:
                 raise GpgRuntimeError(self.context.returncode, _('unable to signing a single key: %s') % e.found().decode('utf-8') + proc.stderr.read())
+        if multiuid:
+            self.context.write(proc.stdin, "save")
         # expect the passphrase confirmation
         try:
             self.context.seek(proc.stderr, 'GOOD_PASSPHRASE')
         except GpgProtocolError:
-            raise GpgRuntimeError(self.context.returncode, _('password confirmation failed'))
+            # GnuPG 2.1 stopped outputing GOOD_PASSPHRASE, so try our luck at just waiting for output
+            self.context.returncode = proc.wait()
+            if self.context.returncode == 0:
+                return True
+            else:
+                raise GpgRuntimeError(self.context.returncode, _('password confirmation failed'))
         if multiuid:
             # we save the resulting key in uid selection mode
             self.context.expect(proc.stderr, 'GET_LINE keyedit.prompt')
-            self.context.write(proc.stdin, "save")
             self.context.expect(proc.stderr, 'GOT_IT')
         return proc.wait() == 0
 
