@@ -15,11 +15,12 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from monkeysign import __version__
+import monkeysign
 # gpg interface
 from monkeysign.gpg import Keyring, TempKeyring, GpgRuntimeError, Context
 import monkeysign.translation
 
+import errno
 # mail functions
 from email.mime.multipart import MIMEMultipart
 from email.mime.message import MIMEMessage
@@ -50,6 +51,26 @@ import socket
 import tempfile
 
 
+class RunTests(argparse._VersionAction):
+    '''argparse Action handler to run tests
+
+    we want to run them immediately because we don't want to force
+    users to provide a pattern.
+
+    '''
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        '''run the test suite and exit with the number of failures'''
+        import unittest
+        sys.argv.remove('--test')
+        if '--version' in sys.argv:
+            sys.argv.remove('--version')
+        testdir = os.path.join(os.path.dirname(__file__), 'tests')
+        suite = unittest.TestLoader().discover(testdir)
+        results = unittest.TextTestRunner().run(suite)
+        parser.exit(len(results.errors))
+
+
 class MonkeysignArgumentParser(argparse.ArgumentParser):
     # config files, in priority order (first is higher)
     configs = ['~/.config/monkeysign.conf', '/etc/monkeysign.conf']
@@ -60,7 +81,7 @@ class MonkeysignArgumentParser(argparse.ArgumentParser):
 # comments will be lost on next write"""
 
     # those parameters will not be saved to the config file
-    ephemeral = ['save', 'version', 'help', 'pattern', 'to']
+    ephemeral = ['save', 'version', 'help', 'pattern', 'to', 'test']
 
     def __init__(self,
                  prog=None, usage=None, description=None, epilog=None,
@@ -91,6 +112,8 @@ class MonkeysignArgumentParser(argparse.ArgumentParser):
                             help=_('explain what we do along the way'))
         self.add_argument('-n', '--dry-run', dest='dryrun', action='store_true',
                             help=_('do not actually do anything'))
+        self.add_argument('--test', action=RunTests,
+                          help=_('run test suite, exit with the number of failing tests'))
         self.add_argument('-u', '--user', nargs='+',
                             help=_('user id to sign the key with (equivalent '
                                    'to GPG\'s --local-user option)'))
@@ -192,6 +215,8 @@ class MonkeysignArgumentParser(argparse.ArgumentParser):
             args.user = "".join(args.user)
         if args.save:
             self.save_config(args)
+        if args.local:
+            args.nomail = True
         return args
 
     def save_config(self, args, config=None, close=True):
@@ -300,7 +325,12 @@ configuration files are parsed by default: %s"""
                     % (distribution, platform.system()))
         info.append('PID: %d, CWD: %s' % (os.getpid(), os.getcwd()))
         info.append('Command: %r' % sys.argv)
-        info.append('GnuPG: %s' % Context().version())
+        try:
+            info.append('GnuPG: %s' % Context().version())
+        except OSError as e:
+            if e.errno == errno.ENOENT:
+                info.append('GnuPG: error! %s: %s'
+                            % (Context.gpg_binary, str(e)))
         return info
 
     def __init__(self, args = None):
@@ -372,6 +402,23 @@ configuration files are parsed by default: %s"""
             if e.errno != errno.ENOENT:
                 raise
 
+        # install the gpg agent socket for GnuPG 2.1 because
+        # --secret-keyring silently fails
+        # this is apparently how we should do things:
+        # https://lists.gnupg.org/pipermail/gnupg-devel/2015-January/029301.html
+        # cargo-culted from caff, thanks guilhem!
+        src = self.keyring.get_agent_socket()
+        dst = self.tmpkeyring.get_agent_socket()
+        self.log(_('installing symlinks for sockets from %s to %s') % (src, dst))
+        try:
+            os.unlink(dst)
+        except OSError as e:
+            if e.errno == errno.ENOENT:
+                pass
+            else:
+                raise
+        os.symlink(src, dst)
+
     def main(self):
         """
         General process
@@ -606,7 +653,7 @@ expects an EmailFactory email, but will not mail if nomail is set"""
                                          stderr=subprocess.PIPE)  # nosec
                 except OSError as e:
                     self.warn(_('cannot find MTA %s, try specifying --mua, --mta or --smtp: %s')
-                              % (command, e))
+                              % (self.options.mua, repr(e)))
                     return False
 
                 if self.options.mua:
@@ -773,7 +820,10 @@ mailto: who to send the mail to (usually similar to recipient, but can be used t
         p2.set_payload(encrypted)
         msg = MIMEMultipart('encrypted', None, [p1, p2], protocol="application/pgp-encrypted")
         msg.preamble = _('This is a multi-part message in OpenPGP/MIME format...')
-        msg['Subject'] = Header(self.subject.encode('utf-8'), 'UTF-8').encode()
+        try:
+            msg['Subject'] = Header(self.subject.encode('ascii'))
+        except UnicodeEncodeError:
+            msg['Subject'] = Header(self.subject, 'UTF-8')
         name, address = parseaddr(self.mailfrom)
         msg['From'] = formataddr((Header(name.encode('utf-8'), 'UTF-8').encode(), address))
         name, address = parseaddr(self.mailto)
